package com.dayou.service.impl;

import com.dayou.dto.ItemDTO;
import com.dayou.entity.Item;
import com.dayou.entity.ItemUser;
import com.dayou.mapper.ItemMapper;
import com.dayou.mapper.ItemUserMapper;
import com.dayou.service.IItemService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.service.IItemUserService;
import com.dayou.utils.LoginContext;
import com.dayou.vo.ItemStatVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.dayou.utils.ExcelUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;
import java.util.List;
import java.util.ArrayList;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.transaction.annotation.Transactional;
import com.dayou.enums.BatchTaskTypeEnum;

/**
 * <p>
 * 项目信息表 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2022-12-08
 */
@Service
public class ItemServiceImpl extends ServiceImpl<ItemMapper, Item> implements IItemService {

    @Autowired
    private IItemUserService iItemUserService;

    @Autowired
    private ItemMapper itemMapper;

    @Override
    @SuppressWarnings("unchecked")
    public Page<Item> selectPage(Page page,Item item,Boolean me){
        Long userId = null;
        if (me){
            userId = LoginContext.getUserId();
        }
        return itemMapper.getPage(page,item,userId);
    }


    @Override
    public ItemDTO detail(Long id){
        ItemDTO itemDTO = itemMapper.xDetail(id);
        return itemDTO;
    }

    @Transactional
    @Override
    public Boolean add(ItemDTO itemDTO){
        this.save(itemDTO);
        List<Long> userIds = itemDTO.getUserIds();
        Set<ItemUser> itemUsers = userIds.stream().map(x -> {
            ItemUser itemUser = new ItemUser();
            itemUser.setItemId(itemDTO.getId());
            itemUser.setUserId(x);
            return itemUser;
        }).collect(Collectors.toSet());
        return  iItemUserService.saveBatch(itemUsers);
    }

    @Transactional
    @Override
    public Boolean update(ItemDTO itemDTO){
        this.updateById(itemDTO);
        List<Long> userIds = itemDTO.getUserIds();
        iItemUserService.remove(new LambdaQueryWrapper<ItemUser>().eq(ItemUser::getItemId,itemDTO.getId()));
        Set<ItemUser> itemUsers = userIds.stream().map(x -> {
            ItemUser itemUser = new ItemUser();
            itemUser.setItemId(itemDTO.getId());
            itemUser.setUserId(x);
            return itemUser;
        }).collect(Collectors.toSet());
        return  iItemUserService.saveBatch(itemUsers);
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }

    @Override
    public ItemStatVO stat() {
        return itemMapper.stat();
    }
}
